<!DOCTYPE html>
<html lang="en">

<head>
    <meta charset="utf-8">
    <title>copyparty</title>
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=0.8">
    <link rel="stylesheet" media="screen" href="/.cpr/splash.css?_={{ ts }}">
</head>

<body>
    <div id="wrap">
        <p>hello {{ this.uname }}</p>

        {%- if avol %}
        <h1>admin panel:</h1>
        <table><tr><td> <!-- hehehe -->
            <table class="num">
                <tr><td>scanning</td><td>{{ scanning }}</td></tr>
                <tr><td>hash-q</td><td>{{ hashq }}</td></tr>
                <tr><td>tag-q</td><td>{{ tagq }}</td></tr>
                <tr><td>mtp-q</td><td>{{ mtpq }}</td></tr>
            </table>
        </td><td>
            <table class="vols">
                <thead><tr><th>vol</th><th>action</th><th>status</th></tr></thead>
                <tbody>
                    {% for mp in avol %}
                    {%- if mp in vstate and vstate[mp] %}
                    <tr><td><a href="{{ mp }}{{ url_suf }}">{{ mp }}</a></td><td><a href="{{ mp }}?scan">rescan</a></td><td>{{ vstate[mp] }}</td></tr>
                    {%- endif %}
                    {% endfor %}
                </tbody>
            </table>
        </td></tr></table>
        <div class="btns">
            <a href="/?stack">dump stack</a>
        </div>
        {%- endif %}

        {%- if rvol %}
        <h1>you can browse these:</h1>
        <ul>
            {% for mp in rvol %}
            <li><a href="{{ mp }}{{ url_suf }}">{{ mp }}</a></li>
            {% endfor %}
        </ul>
        {%- endif %}

        {%- if wvol %}
        <h1>you can upload to:</h1>
        <ul>
            {% for mp in wvol %}
            <li><a href="{{ mp }}{{ url_suf }}">{{ mp }}</a></li>
            {% endfor %}
        </ul>
        {%- endif %}

        <h1>login for more:</h1>
        <ul>
            <form method="post" enctype="multipart/form-data" action="/">
                <input type="hidden" name="act" value="login" />
                <input type="password" name="cppwd" />
                <input type="submit" value="Login" />
            </form>
        </ul>
    </div>
    <script>

if (localStorage.getItem('lightmode') != 1)
    document.documentElement.setAttribute("class", "dark");

</script>
</body>
</html>