<!DOCTYPE html><html><head>
	<meta charset="utf-8">
	<title>📝🎉 {{ title }}</title>
	<meta http-equiv="X-UA-Compatible" content="IE=edge">
	<meta name="viewport" content="width=device-width, initial-scale=0.7">
	<link rel="stylesheet" href="/.cpr/ui.css?_={{ ts }}">
	<link rel="stylesheet" href="/.cpr/mde.css?_={{ ts }}">
	<link rel="stylesheet" href="/.cpr/deps/mini-fa.css?_={{ ts }}">
	<link rel="stylesheet" href="/.cpr/deps/easymde.css?_={{ ts }}">
</head>
<body>
	<div id="mw">
		<div id="mn"></div>
		<div id="ml">
			<div style="text-align:center;margin:5em 0">
				<div style="font-size:2em;margin:1em 0">Loading</div>
				if you're still reading this, check that javascript is allowed
			</div>
		</div>
		<div id="m">
			<textarea id="mt" style="display:none" autocomplete="off">{{ md }}</textarea>
		</div>
	</div>
	<script>

var last_modified = {{ lastmod }};
var md_opt = {
	link_md_as_html: false,
	allow_plugins: {{ md_plug }},
	modpoll_freq: {{ md_chk_rate }}
};

var lightswitch = (function () {
	var l = localStorage,
		drk = l.getItem('lightmode') != 1,
		f = function (e) {
if (e) drk = !drk;
document.documentElement.setAttribute("class", drk? "dark":"light");
l.setItem('lightmode', drk? 0:1);
		};
	f();
	return f;
})();

	</script>
    <script src="/.cpr/util.js?_={{ ts }}"></script>
	<script src="/.cpr/deps/easymde.js?_={{ ts }}"></script>
	<script src="/.cpr/mde.js?_={{ ts }}"></script>
</body></html>
